import wx
import flags_iso3166svg as flags
try:
    from datetime import datetime, timezone
    from zoneinfo import ZoneInfo               # zoneinfo introduced with python3.9
    tzones = True
except Exception as e:
    tzones = False


class MyFrame(wx.Frame):
    def __init__(self, parent, timestamp=1):
        wx.Frame.__init__(self, None, wx.ID_ANY, "")
        self.SetMinSize = ((1300, 320))

        view_panel = wx.Panel(self, wx.ID_ANY)

        self.choices = {"Australia":"AU","Canada":"CA","Germany":"DE","Spain":"ES","France":"FR",\
                        "United Kingdom":"GB","Italy":"IT","Japan":"JP",\
                        "Russia":"RU","United States of America":"US"}

        self.cbx = wx.ComboBox(view_panel, -1, value = "", size=(300,30), choices=list(self.choices), style=wx.CB_READONLY)
        self.cbx.SetHint("Choose a Country")
        self.cbx.Bind(wx.EVT_COMBOBOX, self.OnSelect)

        self.listCtrl = wx.ListCtrl(view_panel, -1, style=wx.LC_REPORT)
        self.listCtrl.SetMinSize((1100, 400))
        self.listCtrl.InsertColumn(1, "Country")
        self.listCtrl.InsertColumn(2, "Region")
        self.listCtrl.InsertColumn(3, "Code")
        self.listCtrl.InsertColumn(4, "Aka")
        self.listCtrl.InsertColumn(5, "Local Time")
        self.listCtrl.SetColumnWidth(0, 300)
        self.listCtrl.SetColumnWidth(1, 300)
        self.listCtrl.SetColumnWidth(3, 150)
        self.listCtrl.SetColumnWidth(4, 200)
        self.listCtrl.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnListCtrl)
        sizer1 = wx.BoxSizer(wx.VERTICAL)
        sizer1.Add(self.cbx, 0, wx.ALL, 10)
        sizer1.Add(self.listCtrl, 1, wx.EXPAND, 10)
        view_panel.SetSizerAndFit(sizer1)
        mainsizer = wx.BoxSizer(wx.VERTICAL)
        mainsizer.Add(view_panel, 1, flag=wx.ALL|wx.EXPAND, border=5)
        self.SetSizerAndFit(mainsizer)
        self.listCtrl.SetFocus()
        self.Layout()

        self.ico = flags.catalog['Flags'].GetIcon()
        self.SetIcon(self.ico)

        self.Show()

    def OnSelect(self, event):
        Choice = self.choices[event.GetString()]
        self.GetData(Choice)

    def GetData(self, choice):
        self.listCtrl.DeleteAllItems()

        # Turn cairo Off, even if it's available, as it's slow loading all the images in one go
        #flags.Use_Cairo = False


        # pre-load images for the listctrl
        #self.imagelist = wx.ImageList(23, 15, False, 0)                     # Old method using wx.ImageList
        self.imagelist = []                                                  # New method use wx.BitmapBundle
        # Get list regional codes
        self.data = sorted([item for item in flags.catalog.keys() if len(item) > 3 and item.startswith(choice)])
        if not self.data:
            return
        for i in self.data:
            iso2, iso3, name, regional = flags.lookup.get(i)
            #self.imagelist.Add(flags.catalog[iso2].Bitmap)                               # Old method using wx.ImageList
            self.imagelist.append(wx.BitmapBundle.FromBitmap(flags.catalog[iso2].ScaledBitmap)) # New method
        #self.listCtrl.AssignImageList(self.imagelist, wx.IMAGE_LIST_SMALL)               # Old method
        self.listCtrl.SetSmallImages(self.imagelist)                                      # New method

        b_iso2, b_iso3, base_country, b_regional = flags.lookupbasecountry(i)   # Get the base country for regional code
        self.SetTitle("Fun with Flags of " + base_country)

        # load listctrl
        index = 0
        for i in self.data:                                                          # regional entries
            iso2, iso3, name, regional = flags.lookup.get(i)
            if not regional:
                continue
            alt = timenow = ''
            det = flags.details.get(iso2)
            if det:
                alt = det.get('altname')
            if tzones:                                      # Were datetime and zoneinfo imports successful
                regzone = flags.timezone.get(iso2)          # get regional timezone
                if not regzone:
                    regzone = flags.timezone.get(b_iso2)    # no zone get base country timezone
                if regzone:
                    zone = regzone[0]
                    timenow = datetime.now(ZoneInfo(zone)).strftime('%H:%M:%S - %d %b %Y')

            name = flags.lookupname(i)
            self.listCtrl.Append([base_country, "", iso2, alt, timenow])
            self.listCtrl.SetItem(index, 1, name, imageId=index)
            index += 1
        self.Refresh()

        # Turn cairo On, if it's available, for high quality flags
        # wx.svg fails with some svg files rendering the image only partially
        # for example Gibraltar GI is missing the entire 3rd castle turret and China CN has no yellow stars
        # wx.svg also fails entirely with Inkscape svg files that have base64 embedded images
        # whereas cairo seeems to cope with most things but its slow in comparison.

        flags.Use_Cairo = True

    def OnListCtrl(self, event):
        item = event.GetIndex()
        id = self.listCtrl.GetItem(item, 2)
        FlagPopup(parent=self, flag=id.GetText())


class FlagPopup(wx.Frame):
    def __init__(self, parent, flag="Flags"):
        wx.Frame.__init__(self, parent, wx.ID_ANY, "Full Size Flag - "+flag)
        self.Bind(wx.EVT_CLOSE, self.OnQuit)
        panel = wx.Panel(self)
        panel.Bind(wx.EVT_KEY_DOWN, self.OnKey)
    # Use the png option
        self.flag = wx.GenericStaticBitmap(panel, bitmap=wx.Bitmap.FromPNGData(flags.catalog[flag].Png))
        self.flag.SetScaleMode(2)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.flag, 1, wx.ALL | wx.EXPAND)
        panel.SetSizerAndFit(sizer)
        self.SetToolTip("Try resizing the window\nPress Q to Quit")
        self.Show()

    def OnKey(self, event):
        keycode = event.GetUnicodeKey()
        if chr(keycode) == 'Q':
            self.OnQuit(None)
        event.Skip()

    def OnQuit(self, event=None):
        self.Destroy()


if __name__ == "__main__":
    App = wx.App()
    MyFrame(None)
    App.MainLoop()
